#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>

#define TAMANO_BUFFER 100
#define DEFAULT_COUNT 10
#define DEFAULT_SIZE 120

static void sig_usr(int);

pid_t pid1, pid2;   //el pid1 sera para matlab y el pid2 sera para el ping    
int status;
FILE * fdml;
FILE * fdpg;


/**
    Esta es la funcion principal y se encarga de ralizar el ping grafico.
    Para ello, crea un hijo que realiza un ping y otro hijo que ejecuta
    matlab. Posteriormente lee desde el ping y escribe en matlab por medio
    de pipas.
*/
int main (int argc,char *argv[])
{
    char buffer[TAMANO_BUFFER];
    char *token;
    int pd_pg[2];   //pipa para leer desde el ping
    int pd_ml[2];   //pipa para escribir en matlab
    
    int i = 0;
    int j = 0;
    char *host;
    
    //se verifican los comandos correctos.
    if(argc==2)
        printf("Para finalizar, presione CTRL+C en esta ventana\n");
    else
    {
        printf("MODO DE USO:\n");
        printf("$ %s [HOST]\n\n", argv[0]);
        exit(0);
    }
   
    host = argv[1];
    
    //se asigna memoria para los tokens
    if((token=(char *)malloc(100*sizeof(char)))==NULL)
    {               
        printf("\n Error de memoria al asignar espacio...\n");
        exit(0);
    }
    
    // se crea la pipa del ping.
    if (pipe(pd_pg) < 0) {
        perror("pipe");
        exit(1);
    }

    // se crea la pipa de matlab.
    if (pipe(pd_ml) < 0) {
        perror("pipe");
        exit(1);
    }

    // se crea el proceso hijo que trabajar con matlab.
    if ((pid1 = fork()) < 0) 
    {
        perror("fork");
        exit(1);
    }

    // el proceso hijo que realiza las operaciones con matlab
    if (pid1 == 0) 
    {
        // El "standard input" del hijo se une para leer desde la pipa
        dup2(pd_ml[0], 0);
        dup2(3, 1);
        
        execlp("matlab", "matlab", "-nodesktop", 0);
        perror("exec");
        exit(-1);
    } 
    
    sleep(13);
    // se crea el proceso hijo que trabajar con ping.
    if ((pid2 = fork()) < 0) 
    {
        perror("fork");
        exit(1);
    }

    // el proceso hijo que realiza las operaciones con matlab
    if (pid2 == 0)            
    {
        // el "standard output" del hijo se une para escribir en la pipa del ping
        dup2(pd_pg[1], 1);
        close(pd_pg[0]);    // se cierra la pipa de lectura

        execlp("ping", "ping", host, 0);
        perror("exec");
        exit(-1);
    }  
      
    close(pd_ml[0]); //se cierra la pipa de lectura para maltlab
    close(pd_pg[1]); //se cierra la pipa de escritura para el ping
    fdpg = fdopen(pd_pg[0], "r"); 
    fdml = fdopen(pd_ml[1], "w");

    /* Se intenta atrpar la seal CTRL+C para hacer limpieza
        antes de que el programa termine. No se hizo antes,
        para que los proceso hijos no lo herederan, ademas de que
        no tiene sentido hacer limpieza antes de este punto     */
    if (signal(SIGINT, sig_usr) == SIG_ERR)
        printf("No se pudo atrapar la seal SIGINT");   
    
        
    fprintf(fdml, "x = [ ];\n");    
    fprintf(fdml, "xlim('auto');\n");
    fflush(fdml);    

    fgets(buffer, TAMANO_BUFFER, fdpg);
                    
    while(fgets(buffer, TAMANO_BUFFER, fdpg))
    {
        i = 0;        
        if(token=(char *)strtok(buffer, "=\n"))
            i++;
            
        while(token=(char *)strtok(NULL, "=\n"))
        {
            i++;
            if(i==3)
            {
                j++;
                token = (char *)strtok(NULL, " ");
                fprintf(fdml, "x = [ x %s ];\n", token);
                fprintf(fdml, "y = abs(fft(x));\n", token);
                
                fprintf(fdml, "subplot(2,1,1);\n");
                fprintf(fdml, "stem( x ,'bo-','LineWidth',2);\n");
                fprintf(fdml, "xlabel('Nro de Paquetes');\n");
                fprintf(fdml, "ylabel('Tiempo de Respuesta [ms]');\n");
                fprintf(fdml, "title('-- Ping Grafico --');\n");                
                fflush(fdml);
                                
                fprintf(fdml, "subplot(2,1,2);\n");
                fprintf(fdml, "stem( y ,'bo-','LineWidth',2);\n");
                fprintf(fdml, "xlabel('Frecuencia');\n");
                fprintf(fdml, "ylabel('Amplitud');\n");
                fprintf(fdml, "title('-- FFT del Ping Grafico --');\n");                
                fflush(fdml);
                
                break;
            }
        }
    }

    exit(0);
}


/**
    Esta funcion se encarga de recibir una seal (en este caso,
    recibe los CRTL+C ), para realizar las labores de limpieza.
*/
static void sig_usr(int signo)
{
    if (signo != SIGINT)
    {
        printf("I don't know this signal");
        return;
    }

    
    kill(pid2, SIGINT);
    waitpid(pid2, &status, 0);

    fprintf(fdml, "\n exit\n");
    fflush(fdml);
    waitpid(pid1, &status, 0);    
    
    fclose(fdpg);
    fclose(fdml);
    
    printf("\nGracias por utilizar mi programa\n\n");
    return;
}
